# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.books.__init__.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK GTK3 Books Package."""

# RAMSTK Local Imports
from .modulebook import RAMSTKModuleBook
from .workbook import RAMSTKWorkBook
