# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.models.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK database models package."""

# Third Party Imports
from sqlalchemy.ext.declarative import declarative_base

RAMSTK_BASE = declarative_base()
